@@ -14,7 +14,7 @@
 -- You should have received a copy of the GNU Affero General Public License
 -- along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-{-# LANGUAGE CPP, BangPatterns, RecordWildCards #-}
+{-# LANGUAGE CPP, BangPatterns, RecordWildCards, RankNTypes #-}
 {-# OPTIONS_GHC -fno-warn-deprecations #-}
 import Distribution.Simple.Utils (cabalVersion)
 import Distribution.Simple.Configure
@@ -48,10 +48,10 @@ import Distribution.Simple.LocalBuildInfo (LocalBuildInfo(..),
                                            externalPackageDeps,
                                            withComponentsLBI,
                                            withLibLBI)
-#if CABAL_MAJOR == 1 && CABAL_MINOR <= 22
+#if CABAL_MAJOR == 1 && CABAL_MINOR >= 23
+import Distribution.Simple.LocalBuildInfo (localUnitId)
+#elif CABAL_MAJOR == 1 && CABAL_MINOR <= 22
 import Distribution.Simple.LocalBuildInfo (inplacePackageId)
-#else
-import Distribution.Simple.LocalBuildInfo (localComponentId)
 #endif
 
 import Distribution.Simple.GHC (componentGhcOptions)
@@ -204,8 +204,7 @@ main = do
       return $ Just $ ChResponseFlags $ sort nonDefaultFlags
 
     "write-autogen-files":[] -> do
-       -- calls writeAutogenFiles
-      initialBuildSteps distdir pd lbi v
+      initialBuildStepsForAllComponents distdir pd lbi v
       return Nothing
 
     "compiler-version":[] -> do
@@ -307,10 +306,10 @@ getLibrary pd = unsafePerformIO $ do
   readIORef lr
 
 getLibraryClbi pd lbi = unsafePerformIO $ do
-  lr <- newIORef (error "getLibraryClbi: empty IORef")
+  lr <- newIORef Nothing
 
   withLibLBI pd lbi $ \ lib clbi ->
-      writeIORef lr (lib,clbi)
+      writeIORef lr $ Just (lib,clbi)
 
   readIORef lr
 
@@ -328,6 +327,8 @@ componentsMap lbi v distdir f = do
 
     lr <- newIORef []
 
+    -- withComponentsLBI is deprecated but also exists in very old versions
+    -- it's equivalent to withAllComponentsInBuildOrder in newer versions
     withComponentsLBI pd lbi $ \c clbi -> do
         let bi = componentBuildInfo c
             name = componentNameFromComponent c
@@ -354,12 +355,20 @@ componentOptions' (lbi, v, distdir) inplaceFlag flags rf f = do
 componentOptions (lbi, v, distdir) inplaceFlag flags f =
     componentOptions' (lbi, v, distdir) inplaceFlag flags renderGhcOptions' f
 
-componentNameToCh CLibName = ChLibName
+#if CABAL_MAJOR == 1 && CABAL_MINOR < 25
+componentNameToCh CLibName = ChLibName ""
+#elif CABAL_MAJOR == 1 && CABAL_MINOR >= 25
+componentNameToCh (CLibName n) = ChLibName n
+#endif
 componentNameToCh (CExeName n) = ChExeName n
 componentNameToCh (CTestName n) = ChTestName n
 componentNameToCh (CBenchName n) = ChBenchName n
 
+#if CABAL_MAJOR == 1 && CABAL_MINOR < 25
 componentNameFromComponent (CLib Library {}) = CLibName
+#elif CABAL_MAJOR == 1 && CABAL_MINOR >= 25
+componentNameFromComponent (CLib Library {..}) = CLibName libName
+#endif
 componentNameFromComponent (CExe Executable {..}) = CExeName exeName
 componentNameFromComponent (CTest TestSuite {..}) = CTestName testName
 componentNameFromComponent (CBench Benchmark {..}) = CBenchName benchmarkName
@@ -408,28 +417,34 @@ removeInplaceDeps :: Verbosity
                   -> ComponentLocalBuildInfo
                   -> (ComponentLocalBuildInfo, GhcOptions)
 removeInplaceDeps v lbi pd clbi = let
-    (lib, libclbi) = getLibraryClbi pd lbi
-    libbi = libBuildInfo lib
-    liboutdir = componentOutDir lbi (CLib lib)
-    libopts = (componentGhcOptions normal lbi libbi libclbi liboutdir) {
-                                      ghcOptPackageDBs = []
-#if CABAL_MAJOR == 1 && CABAL_MINOR > 22
-                                    , ghcOptComponentId = NoFlag
-#endif
-                                  }
-
     (ideps, deps) = partition isInplaceDep (componentPackageDeps clbi)
     hasIdeps = not $ null ideps
+    libopts =
+      case getLibraryClbi pd lbi of
+        Just (lib, libclbi) | hasIdeps ->
+          let
+            libbi = libBuildInfo lib
+            liboutdir = componentOutDir lbi (CLib lib)
+          in
+            (componentGhcOptions normal lbi libbi libclbi liboutdir) {
+                ghcOptPackageDBs = []
+#if CABAL_MAJOR == 1 && CABAL_MINOR > 22 && CABAL_MINOR < 23
+              , ghcOptComponentId = NoFlag
+#endif
+
+            }
+        _ -> mempty
     clbi' = clbi { componentPackageDeps = deps }
 
-  in (clbi', if hasIdeps then libopts else mempty)
+  in (clbi', libopts)
 
  where
    isInplaceDep :: (InstalledPackageId, PackageId) -> Bool
-#if CABAL_MAJOR == 1 && CABAL_MINOR <= 22
+#if CABAL_MAJOR == 1 && CABAL_MINOR >= 23
+   isInplaceDep (ipid, pid) = localUnitId lbi == ipid
+#elif CABAL_MAJOR == 1 && CABAL_MINOR <= 22
    isInplaceDep (ipid, pid) = inplacePackageId pid == ipid
-#else
-   isInplaceDep (ipid, pid) = localComponentId lbi == ipid
+
 #endif
 
 
@@ -440,11 +455,29 @@ nubPackageFlags opts = opts
 nubPackageFlags opts = opts { ghcOptPackages = nub $ ghcOptPackages opts }
 #endif
 
+renderGhcOptions' :: LocalBuildInfo
+                  -> Verbosity
+                  -> GhcOptions
+                  -> IO [String]
 renderGhcOptions' lbi v opts = do
 #if CABAL_MAJOR == 1 && CABAL_MINOR < 20
   (ghcProg, _) <- requireProgram v ghcProgram (withPrograms lbi)
   let Just ghcVer = programVersion ghcProg
   return $ renderGhcOptions ghcVer opts
-#else
+#elif CABAL_MAJOR == 1 && CABAL_MINOR >= 20 && CABAL_MINOR < 24
+-- && CABAL_MINOR < 24
   return $ renderGhcOptions (compiler lbi) opts
+#elif CABAL_MAJOR == 1 && CABAL_MINOR >= 24
+--  CABAL_MAJOR == 1 && CABAL_MINOR >= 24
+  return $ renderGhcOptions (compiler lbi) (hostPlatform lbi) opts
+#endif
+
+
+#if CABAL_MAJOR == 1 && CABAL_MINOR < 25
+initialBuildStepsForAllComponents distdir pd lbi v =
+  initialBuildSteps distdir pd lbi v
+#elif CABAL_MAJOR == 1 && CABAL_MINOR >= 25
+initialBuildStepsForAllComponents distdir pd lbi v =
+  withComponentsLBI pd lbi $ \_c clbi ->
+    initialBuildSteps distdir pd lbi clbi v
 #endif
